/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "visualizermodel.h"

#include <QDebug>

#include <qgrpcasyncreply.h>

#include "dense.h"

using namespace remotecontrol;
using namespace QtProtobuf;

VisualizerModel::VisualizerModel(std::shared_ptr<RemoteControlClient> &client, QObject *parent) : QObject(parent)
  , m_client(client)
{
    m_client->getConfiguration({}, this, [this](QGrpcAsyncReply *reply) {
        m_networkConfig = reply->read<Configuration>();
        sizesChanged();
    });

    QObject::connect(client.get(), &remotecontrol::RemoteControlClient::ActivationsUpdated, [](const remotecontrol::LayerMatrix &activations) {
        Dense dense(activations.matrix().matrix());
        qDebug() << "ActivationsUpdated:" << dense.rows() << dense.columns();
    });
    QObject::connect(client.get(), &remotecontrol::RemoteControlClient::BiasesUpdated, [](const remotecontrol::LayerMatrix &biases) {
        Dense dense(biases.matrix().matrix());
        qDebug() << "BiasesUpdated:" << dense.rows() << dense.columns();
    });
    QObject::connect(client.get(), &remotecontrol::RemoteControlClient::WeightsUpdated, [](const remotecontrol::LayerMatrix &weights) {
        Dense dense(weights.matrix().matrix());
        qDebug() << "WeightsUpdated:" << dense.rows() << dense.columns();
    });
    client->subscribeActivationsUpdates({});
    client->subscribeBiasesUpdates({});
    client->subscribeWeightsUpdates({});
}
